@@ -1,13 +1,12 @@
-from numpy import diag, array, sqrt,  eye, ones, inf, any, copy, zeros, dot, where, all, tile, sum, nan, isfinite, float64, isnan, log10, \
-max, sign, array_equal, nonzero, ix_, arctan, pi, logical_not, logical_and, atleast_2d, matrix, delete, empty, ndarray, \
-logical_and, logical_not
-from numpy.linalg import norm, solve, LinAlgError
+from numpy import  inf, any, copy, dot, where, all, nan, isfinite, float64, isnan,  \
+max, sign, array_equal, matrix, delete, ndarray
+from numpy.linalg import norm#, solve, LinAlgError
 
 from openopt.kernel.baseSolver import *
-from openopt.kernel.Point import Point
+#from openopt.kernel.Point import Point
 from openopt.kernel.setDefaultIterFuncs import *
-from UkrOptMisc import getBestPointAfterTurn
-from PolytopProjection import PolytopProjection
+#from openopt.solvers.UkrOpt.UkrOptMisc import getBestPointAfterTurn
+from openopt.solvers.UkrOpt.PolytopProjection import PolytopProjection
 
 class gsubg(baseSolver):
     __name__ = 'gsubg'
@@ -17,6 +16,7 @@ class gsubg(baseSolver):
     __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
     iterfcnConnected = True
     _canHandleScipySparse = True
+    _requiresBestPointDetection = True
 
     #gsubg default parameters
     h0 = 1.0
@@ -29,20 +29,26 @@ class gsubg(baseSolver):
     show_nnan = False
     doBackwardSearch = True
     new_bs = True
-    approach = 'all active'
-    zhurb = 100
+    
+    maxVectors = 100
+    maxShoots = 15
     sigma = 1e-3
+    
     dual = True
-    ls_direction = 'simple'
-    qpsolver = 'cvxopt_qp'
-    ns = 15
-    dilation = 'auto'
     addASG = False
+    
+    # TODO: involve it?
+    useLinePoints = True
 
-    def __init__(self): pass
+    def __init__(self): 
+        self.approach = 'all active'
+        self.qpsolver = 'cvxopt_qp'
+        self.ls_direction = 'simple'
+        self.dilation = 'auto'
+        
     def __solver__(self, p):
         assert self.approach == 'all active'
-        if not p.isUC: p.warn('Handling of constraints is not implemented properly for the solver %s yet' % self.__name__)
+        #if not p.isUC: p.warn('Handling of constraints is not implemented properly for the solver %s yet' % self.__name__)
         
         dilation = self.dilation
         assert dilation in ('auto', True, False, 0, 1)
@@ -87,7 +93,7 @@ class gsubg(baseSolver):
         T = self.T
         # alternatively instead of alp=self.alp etc you can use directly self.alp etc
 
-        n = p.n
+        #n = p.n
         x0 = p.x0
         
         if p.nbeq == 0 or any(abs(p._get_AeqX_eq_Beq_residuals(x0))>p.contol): # TODO: add "or Aeqconstraints(x0) out of contol"
@@ -95,7 +101,7 @@ class gsubg(baseSolver):
             x0[x0>p.ub] = p.ub[x0>p.ub]
         
         hs = asarray(h0, T)
-        ls_arr = []
+        #ls_arr = []
 
         """                         Nikolay G. Zhurbenko generalized epsilon-subgradient engine                           """
         bestPoint = Point(asarray(copy(x0), T))
@@ -103,7 +109,7 @@ class gsubg(baseSolver):
         prevIter_best_ls_point = bestPoint
         best_ls_point = bestPoint
         iterStartPoint = bestPoint
-        prevIter_bestPointAfterTurn = bestPoint
+        #prevIter_bestPointAfterTurn = bestPoint
         bestPointBeforeTurn = None
         g = bestPoint._getDirection(self.approach)
         g1 = iterStartPoint._getDirection(self.approach, currBestFeasPoint = bestFeasiblePoint)
@@ -116,18 +122,18 @@ class gsubg(baseSolver):
         HS = []
         LS = []
         
-        # TODO: add possibility to handle f_opt if known instead of Ftol
-        #Ftol = 1.0
-        if p.Ftol is None:
-            p.warn("""The solver requres user-supplied Ftol (objective function tolerance); 
+        # TODO: add possibility to handle f_opt if known instead of fTol
+        #fTol = 1.0
+        if p.fTol is None:
+            p.warn("""The solver requres user-supplied fTol (objective function tolerance); 
             since you have not provided it value, 15*ftol = %0.1e will be used""" % (15*p.ftol))
-            p.Ftol = 15 * p.ftol
-        Ftol_start = p.Ftol/2.0
-        Ftol = Ftol_start
+            p.fTol = 15 * p.ftol
+        fTol_start = p.fTol/2.0
+        fTol = fTol_start
         
         subGradientNorms, points, values, isConstraint, epsilons, inactive, normedSubGradients, normed_values = [], [], [], [], [], [], [], []
         StoredInfo = [subGradientNorms, points, values, isConstraint, epsilons, inactive, normedSubGradients, normed_values]
-        nMaxVec = self.zhurb
+        nMaxVec = self.maxVectors
         nVec = 0
         ns = 0
         #ScalarProducts = empty((10, 10))
@@ -135,28 +141,29 @@ class gsubg(baseSolver):
         
         
         """                           gsubg main cycle                                    """
-
-        for itn in xrange(1500000):
+        itn = -1
+        while True:
+            itn += 1
             # TODO: change inactive data removing 
             # TODO: change inner cycle condition
             # TODO: improve 2 points obtained from backward line search
             koeffs = None
             
-            while ns < self.ns:
+            while ns < self.maxShoots:
                 
                 ns += 1
                 nAddedVectors = 0
                 projection = None
-                F0 = asscalar(bestFeasiblePoint.f() - Ftol_start) if bestFeasiblePoint is not None else nan
+                F0 = asscalar(bestFeasiblePoint.f() - fTol_start) if bestFeasiblePoint is not None else nan
                 
                 #iterStartPoint = prevIter_best_ls_point
                 if bestPointBeforeTurn is None:
                     sh = schedule = [bestPoint]
                     #x = iterStartPoint.x
                 else:
-                    sh = [point1, point2]
+                    sh = [point1, point2] if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) else [point2, point1]
                     #sh = [iterStartPoint, bestPointBeforeTurn, bestPointAfterTurn]
-                    sh.sort(cmp = lambda point1, point2: -1+2*int(point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint)))
+                    #sh.sort(cmp = lambda point1, point2: -1+2*int(point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint)))
                     iterStartPoint = sh[-1]
                     schedule = [point for point in sh if id(point.x) != id(points[-1])]
                     #x = iterStartPoint.x.copy()
@@ -230,7 +237,8 @@ class gsubg(baseSolver):
                     nVec += 1
                     normedSubGradients.append(tmp/n_tmp)
                     subGradientNorms.append(n_tmp)
-                    val = n_tmp*(1-1e-7) # to prevent small numerical errors accumulation
+                    val = ProjectionVal
+                    #val = n_tmp*(1-1e-7) # to prevent small numerical errors accumulation
                     values.append(asscalar(val))
                     normed_values.append(asscalar(val/n_tmp))# equals to 0
                     epsilons.append(asscalar((val + dot(prevIterPoint.x, tmp))/n_tmp))
@@ -308,13 +316,13 @@ class gsubg(baseSolver):
 
                 
                 indActive = where(valDistances >= 0)[0]
-                m = len(indActive)
+                #m = len(indActive)
                 product = None
 
-                #print('Ftol: %f   m: %d   ns: %d' %(Ftol, m, ns))
+                #print('fTol: %f   m: %d   ns: %d' %(fTol, m, ns))
                 #raise 0
-                if p.debug: p.debugmsg('Ftol: %f     ns: %d' %(Ftol, ns))
-                Projection = None
+                if p.debug: p.debugmsg('fTol: %f     ns: %d' %(fTol, ns))
+                #Projection = None
                 if nVec > 1:
                     normalizedSubGradients = asfarray(normedSubGradients)
                     product = dot(normalizedSubGradients, normalizedSubGradients.T)
@@ -324,7 +332,7 @@ class gsubg(baseSolver):
                     #maxQPshoutouts = 1
                     
                     for j in range(maxQPshoutouts if bestFeasiblePoint is not None else 1):
-                        F = asscalar(bestFeasiblePoint.f() - Ftol * 5**j) if bestFeasiblePoint is not None else nan
+                        F = asscalar(bestFeasiblePoint.f() - fTol * 5**j) if bestFeasiblePoint is not None else nan
                         valDistances2_modified = asfarray([(0 if isConstraint[i] else -F) for i in range(nVec)]) / asfarray(subGradientNorms)
                         ValDistances = valDistances +  valDistances2_modified - valDistances2
                         
@@ -362,12 +370,14 @@ class gsubg(baseSolver):
                             threshold = 1e-9 # for to prevent small numerical issues
                             if j == 0 and any(dot(normalizedSubGradients, projection) < ValDistances * (1-threshold*sign(ValDistances)) - threshold):
                                 p.istop = 16
-                                p.msg = 'optimal solution wrt required Ftol has been obtained'
+                                p.msg = 'optimal solution wrt required fTol = %g has been obtained' % p.fTol
                                 return
                                 
                             #p.debugmsg('g1 shift: %f' % norm(g1/norm(g1)-projection/norm(projection)))
                             g1 = projection
-                            if j == 0: Projection = projection
+                            #if j == 0: 
+                                #Projection = projection
+                                #ProjectionVal = sum(koeffs*asfarray(ValDistances))
                             #hs = 0.4*norm(g1)
                             M = norm(koeffs, inf)
                             # TODO: remove the cycles
@@ -403,6 +413,10 @@ class gsubg(baseSolver):
                     g1 = D.getDilatedVector(g1)
                     #g1 = tmp
                 
+                if not isinstance(g1, ndarray) or isinstance(g1, matrix):
+                    g1 = g1.A
+                
+                g1 = g1.flatten()
                 
                 if any(g1): 
                     g1 /= p.norm(g1)
@@ -418,11 +432,9 @@ class gsubg(baseSolver):
                 
                 hs_cumsum = 0
                 hs_start = hs
-                if not isinstance(g1, ndarray) or isinstance(g1, matrix):
-                    g1 = g1.A.flatten()
-                    
+                
                 hs_mult = 4.0
-                for ls in xrange(p.maxLineSearch):
+                for ls in range(p.maxLineSearch):
                     
 #                    if ls > 20:
 #                        hs_mult = 2.0
@@ -449,7 +461,7 @@ class gsubg(baseSolver):
                     #if not self.checkTurnByGradient:
                     
                     #TODO: create routine for modifying bestFeasiblePoint
-                    if newPoint.isFeas(False) and (bestFeasiblePoint is None or newPoint.f() > bestFeasiblePoint):
+                    if newPoint.isFeas(False) and (bestFeasiblePoint is None or newPoint.f() > bestFeasiblePoint.f()):
                         bestFeasiblePoint = newPoint
                             
                     if newPoint.betterThan(oldPoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
@@ -472,17 +484,17 @@ class gsubg(baseSolver):
 
                 p.debugmsg('ls_forward: %d' %ls)
                 """                          Backward line search                          """
-                maxLS = 500 #if ls == 0 else 5
-                maxDeltaF = p.ftol / 16.0#Ftol/4.0 #p.ftol / 16.0
-                maxDeltaX = p.xtol / 2.0 #if m < 2 else hs / 16.0#Xdist/16.0
+                #maxLS = 500 #if ls == 0 else 5
+                #maxDeltaF = p.ftol / 16.0#fTol/4.0 #p.ftol / 16.0
+                #maxDeltaX = p.xtol / 2.0 #if m < 2 else hs / 16.0#Xdist/16.0
                 
-                ls_backward = 0
+                #ls_backward = 0
                     
                 #DEBUG
-#                print '!!!!1:', isPointCovered(oldoldPoint, newPoint, bestFeasiblePoint, Ftol), '<<<'
-#                print '!!!!2:', isPointCovered(newPoint, oldoldPoint, bestFeasiblePoint, Ftol), '<<<'
-#                print '!!!!3:', isPointCovered(iterStartPoint, newPoint, bestFeasiblePoint, Ftol), '<<<'
-#                print '!!!!4:', isPointCovered(newPoint, iterStartPoint, bestFeasiblePoint, Ftol), '<<<'
+#                print '!!!!1:', isPointCovered(oldoldPoint, newPoint, bestFeasiblePoint, fTol), '<<<'
+#                print '!!!!2:', isPointCovered(newPoint, oldoldPoint, bestFeasiblePoint, fTol), '<<<'
+#                print '!!!!3:', isPointCovered(iterStartPoint, newPoint, bestFeasiblePoint, fTol), '<<<'
+#                print '!!!!4:', isPointCovered(newPoint, iterStartPoint, bestFeasiblePoint, fTol), '<<<'
 #                raise 0
                 #DEBUG END
                 
@@ -490,7 +502,7 @@ class gsubg(baseSolver):
                 maxRecNum = 400#4+int(log2(norm(oldoldPoint.x-newPoint.x)/p.xtol)) 
                 #assert dot(oldoldPoint.df(), newPoint.df()) < 0
                 #assert sign(dot(oldoldPoint.df(), g1)) != sign(dot(newPoint.df(), g1))
-                point1, point2, nLSBackward = LocalizedSearch(oldoldPoint, newPoint, bestFeasiblePoint, Ftol, p, maxRecNum, self.approach)
+                point1, point2, nLSBackward = LocalizedSearch(oldoldPoint, newPoint, bestFeasiblePoint, fTol, p, maxRecNum, self.approach)
                 
                 
                 #assert sign(dot(point1.df(), g1)) != sign(dot(point2.df(), g1))
@@ -530,15 +542,15 @@ class gsubg(baseSolver):
     #            print 'ls_backward', ls_backward
 
     #            if ls_backward < -4:
-    #                Ftol /= 2.0
+    #                fTol /= 2.0
     #            elif ls > 4:
-    #                Ftol *= 2.0
+    #                fTol *= 2.0
     #                
-    #            print 'Ftol:', Ftol
+    #            print 'fTol:', fTol
                 
                 """                                 Updating hs                                 """
                 step_x = p.norm(best_ls_point.x - prevIter_best_ls_point.x)
-                step_f = abs(best_ls_point.f() - prevIter_best_ls_point.f())
+                #step_f = abs(best_ls_point.f() - prevIter_best_ls_point.f())
                 HS.append(hs_start)
                 assert ls >= 0
                 LS.append(ls)
@@ -552,7 +564,7 @@ class gsubg(baseSolver):
 #                        hs = (mean_ls - j0 + 1)**0.5 * hs_start
 #                    else:
 #                        #hs = hs_start / 16.0
-#                        if (ls == 0 and ls_backward == -maxLS) or self.zhurb!=0:
+#                        if (ls == 0 and ls_backward == -maxLS) or self.maxVectors!=0:
 #                            shift_x = step_x / p.xtol
 #                            shift_f = step_f / p.ftol
 #    #                        print 'shift_x: %e    shift_f: %e' %(shift_x, shift_f)
@@ -564,7 +576,7 @@ class gsubg(baseSolver):
 #                                mp = (0.5, (ls/j0) ** 0.5, 1 - 0.2*RD)
 #                                hs *= max(mp)
 
-                prev_hs = hs
+                #prev_hs = hs
                 if step_x != 0: 
                     hs = 0.5*step_x                  
 #                elif ls  == 0 and nLSBackward > 4:
@@ -573,6 +585,9 @@ class gsubg(baseSolver):
 #                    hs *= 2.0
                 else:
                     hs = max((hs / 10.0,  p.xtol/2.0))
+                
+                #if koeffs is not None: hs = sum(koeffs)
+                
                 p.debugmsg('hs after: %0.1e' % hs)
                     #hs = max((p.xtol/100, 0.5*step_x))
                 #print 'step_x:', step_x, 'new_hs:', hs, 'prev_hs:', prev_hs, 'ls:', ls, 'nLSBackward:', nLSBackward
@@ -602,7 +617,7 @@ class gsubg(baseSolver):
                 #if p.debug: assert p.isUC
 
                 
-                prevInnerCycleIterStartPoint = iterStartPoint
+                #prevInnerCycleIterStartPoint = iterStartPoint
                 
                 #if ns > 3: raise 0
                 if best_ls_point_with_start.betterThan(iterStartPoint, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint):
@@ -628,7 +643,7 @@ class gsubg(baseSolver):
 #                        iterStartPoint = best_ls_point_with_start
 #                        
 #                        assert p.isUC
-#                        if iterStartPoint.f() - best_ls_point_with_start.f() > Ftol :                        
+#                        if iterStartPoint.f() - best_ls_point_with_start.f() > fTol :                        
 #                            break
 
 #                    else:
@@ -639,14 +654,17 @@ class gsubg(baseSolver):
             
             isOverHalphPi = product is not None and any(product[indActive].flatten() <= 0)
 
-            if ns == self.ns and isOverHalphPi:
-                p.istop = 16
-                p.msg = 'Max linesearch directions number has been exceeded'
+            if ns == self.maxShoots and isOverHalphPi:
+                p.istop = 17
+                p.msg = '''
+                Max linesearch directions number has been exceeded 
+                (probably solution has been obtained), 
+                you could increase gsubg parameter "maxShoots" (current value: %d)''' % self.maxShoots
                 best_ls_point = best_ls_point_with_start
 
             """                Some final things for gsubg main cycle                """
             prevIter_best_ls_point = best_ls_point_with_start
-            prevIterPoint = iterStartPoint
+            #prevIterPoint = iterStartPoint
             
             
             # TODO: mb move it inside inner loop
@@ -706,13 +724,13 @@ class gsubg(baseSolver):
 #                    s2 = 0
                     
                 if not s2 and any(p.stopdict.values()):
-                    for key,  val in p.stopdict.iteritems():
+                    for key,  val in p.stopdict.items():
                         if val == True:
                             s2 = key
                             break
                 p.istop = s2
                 
-                for key,  val in p.stopdict.iteritems():
+                for key,  val in p.stopdict.items():
                     if key < 0 or key in set([FVAL_IS_ENOUGH, USER_DEMAND_STOP, BUTTON_ENOUGH_HAS_BEEN_PRESSED]):
                         #p.iterfcn(bestPoint)
                         return
@@ -722,12 +740,12 @@ class gsubg(baseSolver):
                     #p.iterfcn(bestPoint)
                     return
 
-isPointCovered2 = lambda pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol, contol:\
-    pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())
+isPointCovered2 = lambda pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol:\
+    pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())
 
-def isPointCovered3(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol, contol):
+def isPointCovered3(pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol):
     if bestFeasiblePoint is not None \
-    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
+    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
         return True
     if not pointWithSubGradient.isFeas(True) and \
     pointWithSubGradient.mr_alt(bestFeasPoint = bestFeasiblePoint) + 1e-15 > \
@@ -735,24 +753,24 @@ def isPointCovered3(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol,
         return True
     return False
 
-def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol, contol):
+def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, fTol, contol):
     #print 'isFeas:', pointWithSubGradient.isFeas(True)
     
 #    if pointWithSubGradient.sum_of_all_active_constraints() != 0 and any(pointWithSubGradient.sum_of_all_active_constraints_gradient()):
 #        return pointWithSubGradient.sum_of_all_active_constraints() + 0.75*contol > \
 #            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.sum_of_all_active_constraints_gradient())
 #    
-#    return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
+#    return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
     
 ##############################
     if pointWithSubGradient.isFeas(True):
         # assert bestFeasiblePoint is not None 
-        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
+        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
     
     elif pointWithSubGradient.sum_of_all_active_constraints() + 0.75*contol > \
     dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.sum_of_all_active_constraints_gradient()):
         return True
-        #return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
+        #return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()) 
     return False
     
 #    return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
@@ -760,14 +778,14 @@ def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol,
 ######################
 #    
 #    if bestFeasiblePoint is not None \
-#    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()): return True
+#    and pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()): return True
 #    
 #    return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
 #            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint))
 ######################
 #    isFeas = pointWithSubGradient.isFeas(True)
 #    if isFeas:
-#        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())
+#        return pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())
 #    else:
 #        return pointWithSubGradient.mr_alt(bestFeasiblePoint = bestFeasiblePoint) + 0.25*contol > \
 #            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint))
@@ -776,7 +794,7 @@ def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol,
 #            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint)) \
 #            if not pointWithSubGradient.isFeas(True) else True
 #    
-#    isCoveredByObjective = pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())\
+#    isCoveredByObjective = pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df())\
 #    if bestFeasiblePoint is not None else True
 #    
 #    return isCoveredByConstraints and isCoveredByObjective
@@ -786,7 +804,7 @@ def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol,
 #            dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient._getDirection('all active', currBestFeasPoint = bestFeasiblePoint)) else False
 #            #, currBestFeasPoint = bestFeasiblePoint)):
 #        
-#    if pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*Ftol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
+#    if pointWithSubGradient.f() - bestFeasiblePoint.f() + 0.75*fTol > dot(pointWithSubGradient.x - pointToCheck.x, pointWithSubGradient.df()):
 #        # if pointWithSubGradient is feas (i.e. not 1st case) than bestFeasiblePoint is not None
 #        return True
 #        
@@ -794,7 +812,7 @@ def isPointCovered4(pointWithSubGradient, pointToCheck, bestFeasiblePoint, Ftol,
 
 isPointCovered = isPointCovered4
 
-def LocalizedSearch(point1, point2, bestFeasiblePoint, Ftol, p, maxRecNum, approach):
+def LocalizedSearch(point1, point2, bestFeasiblePoint, fTol, p, maxRecNum, approach):
 #    bestFeasiblePoint = None
     contol = p.contol
     for i in range(maxRecNum):
@@ -803,12 +821,12 @@ def LocalizedSearch(point1, point2, bestFeasiblePoint, Ftol, p, maxRecNum, appro
 
         new = 0
         if new:
-            if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point2, point1, bestFeasiblePoint, Ftol) \
-            or point2.betterThan(point1, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point1, point2, bestFeasiblePoint, Ftol):
+            if point1.betterThan(point2, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point2, point1, bestFeasiblePoint, fTol) \
+            or point2.betterThan(point1, altLinInEq=True, bestFeasiblePoint = bestFeasiblePoint) and isPointCovered(point1, point2, bestFeasiblePoint, fTol):
                 break
         else:
-            isPoint1Covered = isPointCovered(point2, point1, bestFeasiblePoint, Ftol, contol)
-            isPoint2Covered = isPointCovered(point1, point2, bestFeasiblePoint, Ftol, contol)
+            isPoint1Covered = isPointCovered(point2, point1, bestFeasiblePoint, fTol, contol)
+            isPoint2Covered = isPointCovered(point1, point2, bestFeasiblePoint, fTol, contol)
             #print 'isPoint1Covered:', isPoint1Covered, 'isPoint2Covered:', isPoint2Covered
             if isPoint1Covered and isPoint2Covered:# and i != 0:
                 break
